/* -*- Mode: javascript; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 *	 Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is messagingmenu-extension
 *
 * The Initial Developer of the Original Code is
 * Mozilla Messaging, Ltd.
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Mike Conley <mconley@mozillamessaging.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

var EXPORTED_SYMBOLS = ["LibDbusmenu"];

const LIB_DBUSMENU3 = "libdbusmenu-gtk.so.3";
const LIB_DBUSMENU4 = "libdbusmenu-gtk.so.4";

const Cu = Components.utils;
const Cc = Components.classes;
const Ci = Components.interfaces;

Cu.import("resource://gre/modules/ctypes.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
Cu.import("resource://messagingmenu/LibGObject.jsm");

XPCOMUtils.defineLazyGetter(this, "libdbusmenu", function() {

  var libdbusmenu;
  try {
    libdbusmenu = ctypes.open(LIB_DBUSMENU4);
  } catch(e) {
    // Attempt to fallback to LIB_DBUSMENU3
    libdbusmenu = ctypes.open(LIB_DBUSMENU3);
  }

  if (!libdbusmenu)
    throw "libdbusmenu is unavailable";

  return libdbusmenu;
});

XPCOMUtils.defineLazyGetter(this, "DbusmenuServer", function() {
  return ctypes.StructType("DbusmenuServer");
});

XPCOMUtils.defineLazyGetter(this, "DbusmenuMenuitem", function() {
  return ctypes.StructType("DbusmenuMenuitem");
});

XPCOMUtils.defineLazyGetter(this, "dbusmenu_server_new", function() {
  var dbusmenu_server_new =
    libdbusmenu.declare("dbusmenu_server_new",
                        ctypes.default_abi,
                        DbusmenuServer.ptr,
                        LibGObject.gchar.ptr);
  if (!dbusmenu_server_new)
    throw "dbusmenu_server_new is unavailable";

  return dbusmenu_server_new;
});

XPCOMUtils.defineLazyGetter(this, "dbusmenu_menuitem_new", function() {
  var dbusmenu_menuitem_new =
    libdbusmenu.declare("dbusmenu_menuitem_new",
                        ctypes.default_abi,
                        DbusmenuMenuitem.ptr);
  if (!dbusmenu_menuitem_new)
    throw "dbusmenu_menuitem_new is unavailable";

  return dbusmenu_menuitem_new;
});

XPCOMUtils.defineLazyGetter(this, "dbusmenu_menuitem_property_set", function() {
  var dbusmenu_menuitem_property_set =
    libdbusmenu.declare("dbusmenu_menuitem_property_set",
                        ctypes.default_abi,
                        LibGObject.gboolean,
                        DbusmenuMenuitem.ptr,
                        LibGObject.gchar.ptr,
                        LibGObject.gchar.ptr);
  if (!dbusmenu_menuitem_property_set)
    throw "dbusmenu_menuitem_property_set is unavailable";

  return dbusmenu_menuitem_property_set;
});

XPCOMUtils.defineLazyGetter(this, "dbusmenu_menuitem_property_set_bool", function() {
  var dbusmenu_menuitem_property_set_bool =
    libdbusmenu.declare("dbusmenu_menuitem_property_set_bool",
                        ctypes.default_abi,
                        LibGObject.gboolean,
                        DbusmenuMenuitem.ptr,
                        LibGObject.gchar.ptr,
                        LibGObject.gboolean);
  if (!dbusmenu_menuitem_property_set_bool)
    throw "dbusmenu_menuitem_property_set is unavailable";

  return dbusmenu_menuitem_property_set_bool;
});

XPCOMUtils.defineLazyGetter(this, "dbusmenu_menuitem_child_append", function() {
  var dbusmenu_menuitem_child_append =
    libdbusmenu.declare("dbusmenu_menuitem_child_append",
                        ctypes.default_abi,
                        LibGObject.gboolean,
                        DbusmenuMenuitem.ptr,
                        DbusmenuMenuitem.ptr);
  if (!dbusmenu_menuitem_child_append)
    throw "dbusmenu_menuitem_child_append is unavailable";

  return dbusmenu_menuitem_child_append;
});

XPCOMUtils.defineLazyGetter(this, "dbusmenu_server_set_root", function() {
  var dbusmenu_server_set_root =
    libdbusmenu.declare("dbusmenu_server_set_root",
                        ctypes.default_abi,
                        ctypes.void_t,
                        DbusmenuServer.ptr,
                        DbusmenuMenuitem.ptr);
  if (!dbusmenu_server_set_root)
    throw "dbusmenu_server_set_root is unavailable";

  return dbusmenu_server_set_root;
});

var LibDbusmenu = {
  MENUITEM_SIGNAL_ITEM_ACTIVATED: "item-activated",
  DbusmenuServer: DbusmenuServer,
  DbusmenuMenuitem: DbusmenuMenuitem,
  ServerNew: dbusmenu_server_new,
  MenuitemNew: dbusmenu_menuitem_new,
  MenuitemPropertySet: dbusmenu_menuitem_property_set,
  MenuitemPropertySetBool: dbusmenu_menuitem_property_set_bool,
  MenuitemChildAppend: dbusmenu_menuitem_child_append,
  ServerSetRoot: dbusmenu_server_set_root
};
