/* -*- Mode: javascript; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 *	 Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is messagingmenu-extension
 *
 * The Initial Developer of the Original Code is
 * Mozilla Messaging, Ltd.
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    Mike Conley <mconley@mozillamessaging.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

var EXPORTED_SYMBOLS = ["LibIndicate"];

const APP_DIRECTORY = "/usr/share/applications/";
const LIB_INDICATE = "libindicate.so.5";

const Cu = Components.utils;
const Cc = Components.classes;
const Ci = Components.interfaces;

Cu.import("resource://gre/modules/ctypes.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
Cu.import("resource://messagingmenu/LibGObject.jsm");
Cu.import("resource://messagingmenu/LibDbusmenu.jsm");

XPCOMUtils.defineLazyGetter(this, "libindicate", function() {
  var libindicate = ctypes.open(LIB_INDICATE);
  if (!libindicate)
    throw "libindicate is unavailable";

  return libindicate;
});

XPCOMUtils.defineLazyGetter(this, "IndicateServer", function() {
  return ctypes.StructType("IndicateServer");
});

XPCOMUtils.defineLazyGetter(this, "Indicator", function() {
  return ctypes.StructType("Indicator");
});

XPCOMUtils.defineLazyGetter(this, "indicate_server_ref_default", function() {
  var indicate_server_ref_default =
    libindicate.declare("indicate_server_ref_default",
                        ctypes.default_abi,
                        IndicateServer.ptr);
  if (!indicate_server_ref_default)
    throw "indicate_server_ref_default is unavailable";

  return indicate_server_ref_default;
});

XPCOMUtils.defineLazyGetter(this, "indicate_server_set_type", function() {
  var indicate_server_set_type =
    libindicate.declare("indicate_server_set_type",
                        ctypes.default_abi,
                        ctypes.void_t,
                        IndicateServer.ptr,
                        LibGObject.gchar.ptr);
  if (!indicate_server_set_type)
    throw "indicate_server_set_type is unavailable";

  return indicate_server_set_type;
});

XPCOMUtils.defineLazyGetter(this, "indicate_server_set_desktop_file", function() {
  var indicate_server_set_desktop_file =
    libindicate.declare("indicate_server_set_desktop_file",
                        ctypes.default_abi,
                        ctypes.void_t,
                        IndicateServer.ptr,
                        LibGObject.gchar.ptr);
  if (!indicate_server_set_desktop_file)
    throw "indicate_server_set_desktop_file is unavailable";

  return indicate_server_set_desktop_file;
});

XPCOMUtils.defineLazyGetter(this, "indicate_server_show", function() {
  var indicate_server_show =
    libindicate.declare("indicate_server_show",
                        ctypes.default_abi,
                        ctypes.void_t,
                        IndicateServer.ptr);
  if (!indicate_server_show)
    throw "indicate_server_show is unavailable";

  return indicate_server_show;
});

XPCOMUtils.defineLazyGetter(this, "indicate_server_set_menu", function() {
  var indicate_server_set_menu =
    libindicate.declare("indicate_server_set_menu",
                        ctypes.default_abi,
                        ctypes.void_t,
                        IndicateServer.ptr,
                        LibDbusmenu.DbusmenuServer.ptr);
  if (!indicate_server_set_menu)
    throw "indicate_server_set_menu is unavailable";

  return indicate_server_set_menu;
});

XPCOMUtils.defineLazyGetter(this, "indicate_indicator_new", function() {
  var indicate_indicator_new =
    libindicate.declare("indicate_indicator_new",
                        ctypes.default_abi,
                        Indicator.ptr)
  if (!indicate_indicator_new)
    throw "indicate_indicator_new is unavailable";

  return indicate_indicator_new;
});

XPCOMUtils.defineLazyGetter(this, "indicate_indicator_set_property", function() {
  var indicate_indicator_set_property =
    libindicate.declare("indicate_indicator_set_property",
                        ctypes.default_abi,
                        ctypes.void_t,
                        Indicator.ptr,
                        LibGObject.gchar.ptr,
                        LibGObject.gchar.ptr)
  if (!indicate_indicator_set_property)
    throw "indicate_indicator_set_property is unavailable";

  return indicate_indicator_set_property;
});

XPCOMUtils.defineLazyGetter(this, "indicate_indicator_get_property", function() {
  var indicate_indicator_get_property =
    libindicate.declare("indicate_indicator_get_property",
                        ctypes.default_abi,
                        LibGObject.gchar.ptr,
                        Indicator.ptr,
                        LibGObject.gchar.ptr)
  if (!indicate_indicator_get_property)
    throw "indicate_indicator_get_property is unavailable";

  return indicate_indicator_get_property;
});

XPCOMUtils.defineLazyGetter(this, "indicate_indicator_show", function() {
  var indicate_indicator_show =
    libindicate.declare("indicate_indicator_show",
                        ctypes.default_abi,
                        ctypes.void_t,
                        Indicator.ptr)
  if (!indicate_indicator_show)
    throw "indicate_indicator_show is unavailable";

  return indicate_indicator_show;
});

XPCOMUtils.defineLazyGetter(this, "indicate_indicator_hide", function() {
  var indicate_indicator_hide =
    libindicate.declare("indicate_indicator_hide",
                        ctypes.default_abi,
                        ctypes.void_t,
                        Indicator.ptr)
  if (!indicate_indicator_hide)
    throw "indicate_indicator_hide is unavailable";

  return indicate_indicator_hide;
});

XPCOMUtils.defineLazyGetter(this, "indicate_indicator_is_visible", function() {
  var indicate_indicator_is_visible =
    libindicate.declare("indicate_indicator_is_visible",
                        ctypes.default_abi,
                        LibGObject.gboolean,
                        Indicator.ptr)
  if (!indicate_indicator_is_visible)
    throw "indicate_indicator_is_visible is unavailable";

  return indicate_indicator_is_visible;
});

var LibIndicate = {
  Indicator: Indicator,
  IndicateServer: IndicateServer,
  INDICATOR_MESSAGES_SERVER_TYPE: "message",
  INDICATOR_MESSAGES_PROP_NAME: "name",
  INDICATOR_MESSAGES_PROP_ICON: "icon",
  INDICATOR_MESSAGES_PROP_COUNT: "count",
  INDICATOR_MESSAGES_PROP_TIME: "time",
  INDICATOR_MESSAGES_PROP_ATTENTION: "draw-attention",

  IndicatorPtrCast: function LI_IndicatorPtrCast(aPointer) {
    return ctypes.cast(aPointer, Indicator.ptr);
  },

  ServerRefDefault: indicate_server_ref_default,
  ServerSetType: indicate_server_set_type,
  ServerSetDesktopFile: indicate_server_set_desktop_file,
  ServerShow: indicate_server_show,
  ServerSetMenu: indicate_server_set_menu,
  IndicatorNew: indicate_indicator_new,
  IndicatorSetProperty: indicate_indicator_set_property,
  IndicatorGetProperty: indicate_indicator_get_property,
  IndicatorShow: indicate_indicator_show,
  IndicatorHide: indicate_indicator_hide,
  IndicatorIsVisible: indicate_indicator_is_visible
};
